-- Author      : ketch
-- Create Date : 12/1/2020 7:58:37 PM

local addonname, NS_WishLists_Master = ...
local _G = getfenv(0)
local NS_WishLists_Master =_G.NS_WishLists_Master
local SetGuildRosterShowOffline, GetNumGroupMembers, GetNumGuildMembers, GetGuildRosterInfo = SetGuildRosterShowOffline, GetNumGroupMembers, GetNumGuildMembers, GetGuildRosterInfo

local clr_lines = {
    r =0.2,
    g = 0.2,
    b = 0.4,
    a = 0.2
}
local drk_lines = {
    r =0.4,
    g = 0.4,
    b = 0.8,
    a = 0.2
}
local function IsEquipableItem(itemClassID,itemSubClassID, myClass)
    --print(myClass)
    if itemClassID == 4 then
        for k,v in ipairs(NS_WishLists_Master.ClassArmorType[myClass]) do
            if itemSubClassID == k then 
                return true
            end
        end
    elseif itemClassID == 2 then
        for k,v in ipairs(NS_WishLists_Master.ClassWeaponType[myClass]) do
            if itemSubClassID == k then 
                return true
            end
        end
    else
        return true
    end
    return false
end
function SaveItemInDB(item)
    NS_WishLists_Master_Data_001["ItemsDB"][item.itemID] = item
end

function IsOverQuality(instance_id, quality)
    local q = NS_WishLists_Master_Data_001["config"][instance_id]["itemQuality"]
    if quality >= q or q == nil or q==0 then 
        return true
    end
    return false
end

local itemSlotTable = {
  -- Source: http://wowwiki.wikia.com/wiki/ItemEquipLoc
  ["INVTYPE_AMMO"] =           { 0 },
  ["INVTYPE_HEAD"] =           { 1 },
  ["INVTYPE_NECK"] =           { 2 },
  ["INVTYPE_SHOULDER"] =       { 3 },
  ["INVTYPE_BODY"] =           { 4 },
  ["INVTYPE_CHEST"] =          { 5 },
  ["INVTYPE_ROBE"] =           { 5 },
  ["INVTYPE_WAIST"] =          { 6 },
  ["INVTYPE_LEGS"] =           { 7 },
  ["INVTYPE_FEET"] =           { 8 },
  ["INVTYPE_WRIST"] =          { 9 },
  ["INVTYPE_HAND"] =           { 10 },
  ["INVTYPE_FINGER"] =         { 11, 12 },
  ["INVTYPE_TRINKET"] =        { 13, 14 },
  ["INVTYPE_CLOAK"] =          { 15 },
  ["INVTYPE_WEAPON"] =         { 16, 17 },
  ["INVTYPE_SHIELD"] =         { 17 },
  ["INVTYPE_2HWEAPON"] =       { 16 },
  ["INVTYPE_WEAPONMAINHAND"] = { 16 },
  ["INVTYPE_WEAPONOFFHAND"] =  { 17 },
  ["INVTYPE_HOLDABLE"] =       { 17 },
  ["INVTYPE_RANGED"] =         { 18 },
  ["INVTYPE_THROWN"] =         { 18 },
  ["INVTYPE_RANGEDRIGHT"] =    { 18 },
  ["INVTYPE_RELIC"] =          { 18 },
  ["INVTYPE_TABARD"] =         { 19 },
  ["INVTYPE_BAG"] =            { 20, 21, 22, 23 },
  ["INVTYPE_QUIVER"] =         { 20, 21, 22, 23 }
};

local function usableSlotID ( itemEquipLoc )
  return itemSlotTable[itemEquipLoc] or nil
end

function GetItemSavedInfos(itemID)
    local item;
    local need_save = false;
    local name ,
          _,
          quality,
          iLevel,
          reqLevel,
          item_type,
          item_subType,
          maxStack,
          equipSlot,
          texture,
          vendorPrice,
          itemClassID,
          itemSubClassID,
          bindType,
          expacID,
          itemSetID,
          isCraftingReagent
    if not NS_WishLists_Master_Data_001["ItemsDB"] then
        NS_WishLists_Master_Data_001["ItemsDB"] = {}
    end
    if not NS_WishLists_Master_Data_001["ItemsDB"][itemID] or not NS_WishLists_Master_Data_001["ItemsDB"][itemID]["name"] then
        NS_WishLists_Master_Data_001["ItemsDB"][itemID] = {}
        name ,
        _,
        quality,
        iLevel,
        reqLevel,
        item_type,
        item_subType,
        maxStack,
        equipSlot,
        texture,
        vendorPrice,
        itemClassID,
        itemSubClassID,
        bindType,
        expacID,
        itemSetID,
        isCraftingReagent = GetItemInfo(itemID);
        if name and quality then
            need_save = true;
        end
    else
        name =              NS_WishLists_Master_Data_001["ItemsDB"][itemID]["name"]             
        quality=            NS_WishLists_Master_Data_001["ItemsDB"][itemID]["quality"]          
        iLevel=             NS_WishLists_Master_Data_001["ItemsDB"][itemID]["iLevel"]           
        reqLevel=           NS_WishLists_Master_Data_001["ItemsDB"][itemID]["reqLevel"]         
        item_type=          NS_WishLists_Master_Data_001["ItemsDB"][itemID]["item_type"]        
        item_subType=       NS_WishLists_Master_Data_001["ItemsDB"][itemID]["item_subType"]     
        maxStack=           NS_WishLists_Master_Data_001["ItemsDB"][itemID]["maxStack"]         
        equipSlot=          NS_WishLists_Master_Data_001["ItemsDB"][itemID]["equipSlot"]        
        texture=            NS_WishLists_Master_Data_001["ItemsDB"][itemID]["texture"]          
        vendorPrice=        NS_WishLists_Master_Data_001["ItemsDB"][itemID]["vendorPrice"]      
        itemClassID=        NS_WishLists_Master_Data_001["ItemsDB"][itemID]["itemClassID"]      
        itemSubClassID=     NS_WishLists_Master_Data_001["ItemsDB"][itemID]["itemSubClassID"]   
        bindType=           NS_WishLists_Master_Data_001["ItemsDB"][itemID]["bindType"]         
        expacID=            NS_WishLists_Master_Data_001["ItemsDB"][itemID]["expacID"]          
        itemSetID=          NS_WishLists_Master_Data_001["ItemsDB"][itemID] ["itemSetID"]        
        isCraftingReagent=  NS_WishLists_Master_Data_001["ItemsDB"][itemID]["isCraftingReagent"]
    end
    
    item = {
        ["name"]                = name,
        ["itemID"]              = itemID,
        ["quality"]             = quality,
        ["iLevel"]              = iLevel,
        ["reqLevel"]            = reqLevel,
        ["item_type"]           = item_type,
        ["item_subType"]        = item_subType,
        ["maxStack"]            = maxStack,
        ["equipSlot"]           = equipSlot,
        ["texture"]             = texture,
        ["vendorPrice"]         = vendorPrice,
        ["itemClassID"]         = itemClassID,
        ["itemSubClassID"]      = itemSubClassID,
        ["bindType"]            = bindType,
        ["expacID"]             = expacID,
        ["itemSetID"]           = itemSetID,
        ["isCraftingReagent"]   = isCraftingReagent
    };
    --need_save = true
    --save new item to db
    if need_save then SaveItemInDB(item) end

    return item
end
function IsTargetSlot(instance_id, equipSlot)
    local eq = NS_WishLists_Master_Data_001["config"][instance_id]["SlotSelected"]

    local equipslotnum = usableSlotID(equipSlot)
    if equipslotnum then
        for k, equipSlot_id in ipairs(equipslotnum) do
            if equipSlot_id == eq or eq == nil or eq ==-1 then 
                return true
            end
        end
    else
        --print(equipSlot)
    end
    return false
end
function IsTargetType(instance_id, itemType)
    local it = NS_WishLists_Master_Data_001["config"][instance_id]["itemType"]
    if itemType == it or it == nil or it ==-1 then 
        return true
    end
    return false
end
function mustBeDisplayed(instance_id, item, my_class)
    if not NS_WishLists_Master_Data_001["config"] then
        NS_WishLists_Master_Data_001["config"] = {}
    end
    if not NS_WishLists_Master_Data_001["config"][instance_id] then
        NS_WishLists_Master_Data_001["config"][instance_id] = {}
    end
    if not NS_WishLists_Master_Data_001["config"][instance_id]["itemQuality"] then
        NS_WishLists_Master_Data_001["config"][instance_id]["itemQuality"] = 0
    end
    if not NS_WishLists_Master_Data_001["config"][instance_id]["SlotSelected"] then
        NS_WishLists_Master_Data_001["config"][instance_id]["SlotSelected"] = -1
    end
    if not NS_WishLists_Master_Data_001["config"][instance_id]["itemType"] then
        NS_WishLists_Master_Data_001["config"][instance_id]["itemType"] = -1
    end
     if NS_WishLists_Master_Data_001["config"][instance_id]["UsableOnly"] == nil then
        NS_WishLists_Master_Data_001["config"][instance_id]["UsableOnly"] = true
     end
    local goodQuality   = IsOverQuality(    instance_id,      item.quality)
    local goodSlot      = IsTargetSlot(     instance_id,      item.equipSlot)
    local goodType      = IsTargetType(     instance_id,      item.itemClassID)
    local equipable     = IsEquipableItem(  item.itemClassID, item.itemSubClassID, my_class)
    local displayEquip = false
    local displayItem = 0
    --no equip and dont want to see noeq
    if equipable == false and NS_WishLists_Master_Data_001["config"][instance_id]["UsableOnly"] == true then
        displayItem = 0
    end
    --not equip and dont care to see noeq
    if equipable == false and NS_WishLists_Master_Data_001["config"][instance_id]["UsableOnly"] == false then
       displayItem = 2
       displayEquip = true
    end
    --equip and dont want to see noeq
    if equipable == true and NS_WishLists_Master_Data_001["config"][instance_id]["UsableOnly"] == true then
        displayItem = 1
        displayEquip = true
    end
    if equipable == true and NS_WishLists_Master_Data_001["config"][instance_id]["UsableOnly"] == false then
       displayItem = 1
       displayEquip = true
    end

    return (goodQuality and goodSlot and goodType and displayEquip), displayItem
end
function reSortWL(wl_table)
    local prio_count = 0
    for k, itemid in pairs(wl_table) do
        prio_count = prio_count + 1
    end
    local temp_wl_table = {}
    local new_index = 1
    for index = 1,prio_count+1 do
        if wl_table["prio_"..index] then
            temp_wl_table["prio_"..new_index] = wl_table["prio_"..index]
            new_index = new_index + 1
        else
            
        end
    end
    wl_table = temp_wl_table
    return wl_table
end

function NS_WishLists_Master:getGuildRoster()
    if not NS_WishLists_Master_Data_001["guild_roster"] then
        NS_WishLists_Master_Data_001["guild_roster"] = {}
    end
    local nummembers, numOnline = GetNumGuildMembers()
    for i = 1, nummembers do
        user = {}
        local name, rank, rankIndex, level, class, zone, note, officernote,
              online, status, classFileName, achievementPoints, achievementRank,
              isMobile, isSoREligible, standingID = GetGuildRosterInfo(i)
        if name then
            user.user_name = NS_WishLists_Master:RemoveRealmPName(name)
            user.user_classe = class
            user.user_classef = classFileName
            user.race = ""
            user.rank = rank
            user.level = level
            user.role = "NC"
            local found = false
            if not NS_WishLists_Master_Data_001["guild_roster"] then NS_WishLists_Master_Data_001["guild_roster"] = {} end
            for idx, usr_ in ipairs(NS_WishLists_Master_Data_001["guild_roster"]) do
                if usr_ then
                    if usr_.user_name == user.user_name or  usr_.user_name == name then
                        NS_WishLists_Master_Data_001["guild_roster"][usr_.user_name] = user
                        usr_found = true
                        break
                    end
                end
            end
            if not usr_found then
                 NS_WishLists_Master_Data_001["guild_roster"][user.user_name] = user
            end
        end
    end
end
function NS_WishLists_Master:DrawGuildRoster(container, instance_id)
    if not NS_WishLists_Master_Data_001["config"] then NS_WishLists_Master_Data_001["config"] = {} end
    if not NS_WishLists_Master_Data_001["config"][instance_id] then NS_WishLists_Master_Data_001["config"][instance_id] = {} end
    if NS_WishLists_Master_Data_001["config"][instance_id]["UsableOnly"] == nil then
        NS_WishLists_Master_Data_001["config"][instance_id]["UsableOnly"] = true
    end
    local backdrop1 = {
        bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
        edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]], edgeSize = 16,
        insets = { left = 4, right = 4, top = 4, bottom = 4 }
    }
    -- ScrollingMessageFrame1
    local playerBank = CreateFrame("FRAME", addonname.."_GplayerBank", container.content)
        playerBank:SetPoint("TOPLEFT",0,-17)
        playerBank:SetHeight(360)
        playerBank:SetWidth(200)

    local Title = playerBank:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
        Title:SetPoint("TOPLEFT",playerBank,"TOPLEFT",10,0)
        Title:SetJustifyH("LEFT")
        Title:SetHeight(19)
        Title:SetWidth(200)
        Title:SetText("Membres de la guilde:")

        -- ScrollingMessageFrame1
        playerBank.scrollMotherFrame = CreateFrame("Frame",  addonname.."_GplayerBank_scrollmother", playerBank)
        playerBank.scrollMotherFrame:SetPoint("TOPLEFT", 0, -20)
        playerBank.scrollMotherFrame:SetHeight(335)
        playerBank.scrollMotherFrame:SetWidth(200)
        playerBank.scrollMotherFrame:SetBackdrop(backdrop1)
        playerBank.scrollMotherFrame:SetBackdropColor(0, 0, 0.1)
        playerBank.scrollMotherFrame:SetBackdropBorderColor(0.8, 0.8, 0.8)

        playerBank.scrollFrame = CreateFrame("ScrollFrame", nil, playerBank.scrollMotherFrame, "UIPanelScrollFrameTemplate")
        playerBank.scrollFrame:SetPoint("TOPLEFT", 5, -8)
        playerBank.scrollFrame:SetPoint("BOTTOMRIGHT", -30, 5)
        playerBank.scrollFrame:EnableMouseWheel(true)
        playerBank.scrollFrame:EnableMouse(true)

        playerBank.scrollbg2 = playerBank.scrollFrame:CreateTexture(nil, "BACKGROUND")
        playerBank.scrollbg2:SetAllPoints(true)
        playerBank.scrollbg2:SetColorTexture(0, 0, 0, 0.9)

        playerBank.cnt = CreateFrame("Frame", addonname.."_GplayerBank_cnt", playerBank.scrollMotherFrame)
        playerBank.cnt:SetPoint("TOPLEFT", playerBank.scrollFrame, "TOPLEFT", 0, 0)
        playerBank.cnt:SetPoint("TOPRIGHT", playerBank.scrollFrame, "TOPRIGHT", 0, 0)
        playerBank.cnt:SetHeight(300)
        playerBank.cnt:SetWidth(200)

        playerBank.scrollFrame:SetScrollChild(playerBank.cnt)

        playerBank.scrollbg3 = playerBank.cnt:CreateTexture(nil, "BACKGROUND")
        playerBank.scrollbg3:SetAllPoints(true)
        playerBank.scrollbg3:SetColorTexture(0, 0, 0, 0.9)
        --menu ou bouton============================================================
        local displayAll = CreateFrame("CheckButton", nil, playerBank, "UICheckButtonTemplate");
            displayAll:SetPoint("TOPLEFT",playerBank.scrollMotherFrame,"BOTTOMLEFT", 0, 2);
            displayAll.text:SetWidth(160);
            displayAll.text:SetText(" tout afficher");
            displayAll.text:SetJustifyH("LEFT")
            displayAll.text:Show();
            displayAll:SetChecked(NS_WishLists_Master_Data_001["config"][instance_id]["displayAll"])

        playerBank.displayAll = displayAll
        -- ============================================================
        pair_ = false
        --************
        ----CMD UI
        local ItemsList = CreateFrame("Frame",addonname.."_MyList", container.content)
        ItemsList:SetSize(451,380);
        ItemsList:SetPoint("TOPLEFT",playerBank.scrollMotherFrame,"TOPRIGHT",20,20)
        ItemsList:SetToplevel(true);

        local c = NS_WishLists_Master.drk_lines
        local labelwhenH = ItemsList:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
            labelwhenH:SetPoint("TOPLEFT", 5,0)
            labelwhenH:SetPoint("TOPRIGHT", -5,0)
            labelwhenH:SetJustifyH("CENTER")
            labelwhenH:SetJustifyV("MIDDLE")
            labelwhenH:SetText("Sélection du joueur")
            labelwhenH:SetHeight(19)
            local highlight1h = ItemsList:CreateTexture(nil, "HIGHLIGHT")
            highlight1h:SetAllPoints(labelwhenH)
            highlight1h:SetColorTexture(0.5, 0.5, 0.9, 0.8)
            highlight1h:SetBlendMode("ADD")
            local highlight1bgh = ItemsList:CreateTexture(nil, "BACKGROUND")
            highlight1bgh:SetAllPoints(labelwhenH)
            highlight1bgh:SetColorTexture(c.r, c.g, c.b, c.a)
            highlight1bgh:SetBlendMode("ADD")

        local backdrop1 = {
            bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
            edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]], edgeSize = 16,
            insets = { left = 4, right = 4, top = 4, bottom = 4 }
        }
        -- ScrollingMessageFrame
        local scrollMotherFrame = CreateFrame("Frame", addonname.."_Instance_"..instance_id.."_MyList_ScrollMotherFrame", ItemsList)
        scrollMotherFrame:SetPoint("TOPLEFT", 0, -25)
        scrollMotherFrame:SetPoint("BOTTOMRIGHT", 0, 30)
        scrollMotherFrame:SetBackdrop(backdrop1)
        scrollMotherFrame:SetBackdropColor(0, 0, 0,0.5)
        scrollMotherFrame:SetBackdropBorderColor(0.8, 0, 0.8)

        local scrollFrame = CreateFrame("ScrollFrame", addonname.."_Instance_"..instance_id.."_MyList_ScrollFrame", scrollMotherFrame, "UIPanelScrollFrameTemplate")
        scrollFrame:SetPoint("TOPLEFT", 5, -8)
        scrollFrame:SetPoint("BOTTOMRIGHT", -30, 5)
        scrollFrame:EnableMouseWheel(true)
        scrollFrame:EnableMouse(true)

        local scrollbg2 = scrollFrame:CreateTexture(nil, "BACKGROUND")
        scrollbg2:SetAllPoints(scrollFrame.ScrollBar)
        scrollbg2:SetColorTexture(0, 0, 0, 0.9)

        ItemsList.cnt = CreateFrame("Frame", addonname.."_Instance_"..instance_id.."_MyList_cnt", scrollFrame)
        ItemsList.cnt:SetPoint("TOPLEFT",scrollMotherFrame, "TOPLEFT", 0, 0)
        ItemsList.cnt:SetWidth(421)
        ItemsList.cnt:SetHeight(300)

        --menu quality bouton===================================================================
        local menuLootQuality = CreateFrame("Frame", nil, ItemsList)
            menuLootQuality:SetSize(60, 26)
            menuLootQuality:SetPoint("TOPLEFT",scrollMotherFrame,"BOTTOMLEFT", 0,-3)

        local tleft = menuLootQuality:CreateTexture(nil, "BACKGROUND")
            tleft:SetSize(25,64)
            tleft:SetPoint("TOPLEFT", -18,18)
            tleft:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
            tleft:SetTexCoord(0, 0.1953125, 0, 1)
        
        local tmid = menuLootQuality:CreateTexture(nil, "BACKGROUND")
            tmid:SetSize(60,64)
            tmid:SetPoint("LEFT", tleft, "RIGHT",0,0)
            tmid:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
            tmid:SetTexCoord(0.1953125, 0.8046875, 0, 1)

        local tright = menuLootQuality:CreateTexture(nil, "BACKGROUND")
            tright:SetSize(25,64)
            tright:SetPoint("LEFT", tmid, "RIGHT",0,0)
            tright:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
            tright:SetTexCoord(0.8046875, 1, 0, 1)

        ItemsList.menuLootQuality = menuLootQuality

        local lootQualityDropmenu_str = ItemsList.menuLootQuality:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
            lootQualityDropmenu_str:SetSize(100,10)
            lootQualityDropmenu_str:SetPoint("LEFT",tleft,"RIGHT",-15, 2)

        ItemsList.lootQualityDropmenu_str = lootQualityDropmenu_str

        ItemsList.menuLootQuality.text = (function (self, value) ItemsList.lootQualityDropmenu_str:SetText(value) end)
        if not NS_WishLists_Master_Data_001["config"][instance_id] then
            NS_WishLists_Master_Data_001["config"][instance_id] = {}
        end
        if not NS_WishLists_Master_Data_001["config"][instance_id]["itemQuality"] then
            NS_WishLists_Master_Data_001["config"][instance_id]["itemQuality"] = 0
        end
        local iv = NS_WishLists_Master_Data_001["config"][instance_id]["itemQuality"]
        if not iv then error("Something is going bad..") end
        local r, g, b, hex = GetItemQualityColor(iv)
        local color = "|c"..hex
        local colorend = "|r"
        ItemsList.menuLootQuality.text(self,(color.._G["ITEM_QUALITY"..iv.. "_DESC"]..colorend) or "Qualities:")
        ItemsList.menuLootQuality:SetScript("OnMouseUp", function(self)
            if ItemsList.lootQualityDropdown then
                ItemsList.lootQualityDropdown:Close()
            end
            if ItemsList.lootSlotDropdown then
                ItemsList.lootSlotDropdown:Close()
            end
            if ItemsList.lootTypeDropdown then
                ItemsList.lootTypeDropdown:Close()
            end
            local lootQualityDropdown = NS_WishLists_Master.AceGUI:Create("Dropdown-Pullout")
            lootQualityDropdown:SetWidth(110)
            local header = NS_WishLists_Master.AceGUI:Create("Dropdown-Item-Header")
            header:SetText("Qualities:")
            lootQualityDropdown:AddItem(header)
            lootQualityDropdown:AddItem(header)
            --lootQualityDropdown = NS_WishLists_Master.AceGUI:Create("Dropdown-Pullout")
            local itemsQuality = {}
            for i= 0, 7 do
                local r, g, b, hex = GetItemQualityColor(i)
                local qualityName = _G["ITEM_QUALITY" .. i .. "_DESC"]
                local colorend = "|r"
                local color = "|c"..hex
                local btn1 = NS_WishLists_Master.AceGUI:Create("Dropdown-Item-Execute")
                btn1:SetText((color.._G["ITEM_QUALITY"..i.. "_DESC"]..colorend))

                btn1:SetCallback("OnClick", function(self)
                    container:ReleaseChildren()
                    --self:SetText(color..qualityName..colorend)
                    if not NS_WishLists_Master_Data_001["config"] then NS_WishLists_Master_Data_001["config"] = {} end
                    if not NS_WishLists_Master_Data_001["config"][instance_id] then NS_WishLists_Master_Data_001["config"][instance_id] = {} end
                    NS_WishLists_Master_Data_001["config"][instance_id]["itemQuality"] = i
                    ItemsList.menuLootQuality.text(self, (color.._G["ITEM_QUALITY"..i.. "_DESC"]..colorend))
                    NS_WishLists_Master:fillTargetWishList(instance_id)
                    lootQualityDropdown:Close()
                end)
                lootQualityDropdown:AddItem(btn1)
            end
            local btn1 = NS_WishLists_Master.AceGUI:Create("Dropdown-Item-Execute")
            btn1:SetText(CLOSE)
            btn1:SetCallback("OnClick", function(self)
                container:ReleaseChildren()
                lootQualityDropdown:Close()
            end)
            lootQualityDropdown:AddItem(btn1)
            lootQualityDropdown:Open("TOP", self, "BOTTOM", 0, 5)
            lootQualityDropdown.frame:SetParent(self:GetParent())
            self:GetParent().lootQualityDropdown = lootQualityDropdown
        end)
        --menu Type bouton=================================================================
        local menuLootType = CreateFrame("Frame", nil, ItemsList)
            menuLootType:SetSize(60, 26)
            menuLootType:SetPoint("TOPLEFT",ItemsList.menuLootQuality,"TOPRIGHT", 40, 0)

        local tleft = menuLootType:CreateTexture(nil, "BACKGROUND")
            tleft:SetSize(25,64)
            tleft:SetPoint("TOPLEFT", -18,18)
            tleft:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
            tleft:SetTexCoord(0, 0.1953125, 0, 1)
        
        local tmid = menuLootType:CreateTexture(nil, "BACKGROUND")
            tmid:SetSize(60,64)
            tmid:SetPoint("LEFT", tleft, "RIGHT",0,0)
            tmid:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
            tmid:SetTexCoord(0.1953125, 0.8046875, 0, 1)

        local tright = menuLootType:CreateTexture(nil, "BACKGROUND")
            tright:SetSize(25,64)
            tright:SetPoint("LEFT", tmid, "RIGHT",0,0)
            tright:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
            tright:SetTexCoord(0.8046875, 1, 0, 1)

        ItemsList.menuLootType = menuLootType

        local lootTypeDropmenu_str = ItemsList.menuLootType:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
            lootTypeDropmenu_str:SetSize(100,10)
            lootTypeDropmenu_str:SetPoint("LEFT",tleft,"RIGHT",-15, 2)

        ItemsList.lootTypeDropmenu_str = lootTypeDropmenu_str
        local TypeName1 = {[-1] = "Tous..", [2] = "Armes", [4] = "Armures"}
        ItemsList.menuLootType.text = (function (self, value) ItemsList.lootTypeDropmenu_str:SetText(value) end)
        text_ = "Types:"
        for k,v in pairs(TypeName1) do
            if k == NS_WishLists_Master_Data_001["config"][instance_id]["itemType"] then
                text_ = v
                break
            end
        end
        ItemsList.menuLootType.text(self,text_)
        ItemsList.menuLootType:SetScript("OnMouseUp", function(self)
            if ItemsList.lootQualityDropdown then
                ItemsList.lootQualityDropdown:Close()
            end
            if ItemsList.lootSlotDropdown then
                ItemsList.lootSlotDropdown:Close()
            end
            if ItemsList.lootTypeDropdown then
                ItemsList.lootTypeDropdown:Close()
            end
            local lootTypeDropdown = NS_WishLists_Master.AceGUI:Create("Dropdown-Pullout")
            lootTypeDropdown:SetWidth(110)
            local header = NS_WishLists_Master.AceGUI:Create("Dropdown-Item-Header")
            header:SetText(NS_WishLists_Master_Data_001["config"][instance_id]["itemType"] or "Types:")
            lootTypeDropdown:AddItem(header)
            lootTypeDropdown = NS_WishLists_Master.AceGUI:Create("Dropdown-Pullout")
            local itemsType = {}
            for idx_type, type_ in pairs(TypeName1) do
                local color = "|cFFAAAAAA"
                local btn1 = NS_WishLists_Master.AceGUI:Create("Dropdown-Item-Execute")
                btn1:SetText(color..type_..colorend)
                btn1:SetCallback("OnClick", function(self)
                    container:ReleaseChildren()
                    self:SetText(color..type_..colorend)
                    if not NS_WishLists_Master_Data_001["config"] then NS_WishLists_Master_Data_001["config"] = {} end
                    if not NS_WishLists_Master_Data_001["config"][instance_id] then NS_WishLists_Master_Data_001["config"][instance_id] = {} end
                    NS_WishLists_Master_Data_001["config"][instance_id]["itemType"] = idx_type
                    ItemsList.menuLootType.text(self, color..type_..colorend)
                    NS_WishLists_Master:fillTargetWishList(instance_id)
                    lootTypeDropdown:Close()
                end)
                lootTypeDropdown:AddItem(btn1)
            end
            lootTypeDropdown:Open("TOP", self, "BOTTOM", 0, 5)
            lootTypeDropdown.frame:SetParent(self:GetParent())
            ItemsList.lootTypeDropdown = lootTypeDropdown
        end)
        ItemsList:SetScript("OnEnter", function(self)
            if ItemsList.lootTypeDropdown then
                ItemsList.lootTypeDropdown:Close()
            end
        end)
        --menu slot bouton============================================================
        local menuLootSlot = CreateFrame("Frame", nil, ItemsList)
            menuLootSlot:SetSize(60, 26)
            menuLootSlot:SetPoint("TOPLEFT",ItemsList.menuLootType,"TOPRIGHT", 40, 0)

        local tleft = menuLootSlot:CreateTexture(nil, "BACKGROUND")
            tleft:SetSize(25,64)
            tleft:SetPoint("TOPLEFT", -18,18)
            tleft:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
            tleft:SetTexCoord(0, 0.1953125, 0, 1)
        
        local tmid = menuLootSlot:CreateTexture(nil, "BACKGROUND")
            tmid:SetSize(60,64)
            tmid:SetPoint("LEFT", tleft, "RIGHT",0,0)
            tmid:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
            tmid:SetTexCoord(0.1953125, 0.8046875, 0, 1)

        local tright = menuLootSlot:CreateTexture(nil, "BACKGROUND")
            tright:SetSize(25,64)
            tright:SetPoint("LEFT", tmid, "RIGHT",0,0)
            tright:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
            tright:SetTexCoord(0.8046875, 1, 0, 1)

        ItemsList.menuLootSlot = menuLootSlot

        local lootSlotDropmenu_str = ItemsList.menuLootSlot:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
            lootSlotDropmenu_str:SetSize(100,10)
            lootSlotDropmenu_str:SetPoint("LEFT",tleft,"RIGHT",-15, 2)

        ItemsList.lootSlotDropmenu_str = lootSlotDropmenu_str

        ItemsList.menuLootSlot.text = (function (self, value) ItemsList.lootSlotDropmenu_str:SetText(value) end)
        text_ = "Slots:"
        for idx_slot, slot_text in pairs(NS_WishLists_Master.Slots) do
            if idx_slot == NS_WishLists_Master_Data_001["config"][instance_id]["SlotSelected"] then
                text_ = slot_text
                break
            end
        end
        ItemsList.menuLootSlot.text(self,text_)
        ItemsList.menuLootSlot:SetScript("OnMouseUp", function(self)
            if ItemsList.lootQualityDropdown then
                ItemsList.lootQualityDropdown:Close()
            end
            if ItemsList.lootSlotDropdown then
                ItemsList.lootSlotDropdown:Close()
            end
            if ItemsList.lootTypeDropdown then
                ItemsList.lootTypeDropdown:Close()
            end
            local lootSlotDropdown = NS_WishLists_Master.AceGUI:Create("Dropdown-Pullout")
            lootSlotDropdown:SetWidth(110)
            local header = NS_WishLists_Master.AceGUI:Create("Dropdown-Item-Header")
            header:SetText(NS_WishLists_Master_Data_001["config"][instance_id]["SlotSelected"] or "Slots:")
            lootSlotDropdown:AddItem(header)
            lootSlotDropdown = NS_WishLists_Master.AceGUI:Create("Dropdown-Pullout")
            local itemsSlot = {}
            for idx_slot, slot_text in pairs(NS_WishLists_Master.Slots) do
                local btn1 = NS_WishLists_Master.AceGUI:Create("Dropdown-Item-Execute")
                btn1:SetText(slot_text)
                btn1:SetCallback("OnClick", function(self)     
                    container:ReleaseChildren()
                    self:SetText(slot_text)
                    if not NS_WishLists_Master_Data_001["config"] then NS_WishLists_Master_Data_001["config"] = {} end
                    if not NS_WishLists_Master_Data_001["config"][instance_id] then NS_WishLists_Master_Data_001["config"][instance_id] = {} end
                    NS_WishLists_Master_Data_001["config"][instance_id]["SlotSelected"] = idx_slot
                    ItemsList.menuLootSlot.text(self, slot_text)
                    lootSlotDropdown:Close()
                    NS_WishLists_Master:fillTargetWishList(instance_id)
                end)
                lootSlotDropdown:AddItem(btn1)
            end
            lootSlotDropdown:Open("TOP", self, "BOTTOM", 0, 5)
            lootSlotDropdown.frame:SetParent(self:GetParent())
            self:GetParent().lootSlotDropdown = lootSlotDropdown
        end)
        --menu ou bouton============================================================
        local onlyUsable = CreateFrame("CheckButton", nil, ItemsList, "UICheckButtonTemplate");
            onlyUsable:SetPoint("TOPLEFT",ItemsList.menuLootSlot,"TOPRIGHT", 40, 2);
            onlyUsable.text:SetWidth(160);
            onlyUsable.text:SetText(" Equipable");
            onlyUsable.text:SetJustifyH("LEFT")
            onlyUsable.text:Show();
            onlyUsable:SetChecked(NS_WishLists_Master_Data_001["config"][instance_id]["UsableOnly"])
            onlyUsable:SetScript("OnMouseUp", function(self)
                self:SetChecked(self:GetChecked());
                NS_WishLists_Master_Data_001["config"][instance_id]["UsableOnly"] = not NS_WishLists_Master_Data_001["config"][instance_id]["UsableOnly"]
                NS_WishLists_Master:fillTargetWishList(instance_id)
            end)
        ItemsList.onlyUsable = onlyUsable
        -- ============================================================
        scrollFrame:SetScrollChild(ItemsList.cnt)

        ItemsList.closebutton = closebutton
        ItemsList.scrollFrame = scrollFrame
        ItemsList.scrollMotherFrame = scrollMotherFrame

        local FillGuildRoster = function(self)
            pair_ = false
            if playerBank then
                playerBank.cnt:Hide()
                playerBank.cnt = CreateFrame("Frame", addonname.."_GplayerBank_cnt", playerBank.scrollMotherFrame)
                playerBank.cnt:SetPoint("TOPLEFT", playerBank.scrollFrame, "TOPLEFT", 0, 0)
                playerBank.cnt:SetPoint("TOPRIGHT", playerBank.scrollFrame, "TOPRIGHT", 0, 0)
                playerBank.cnt:SetHeight(300)
                playerBank.cnt:SetWidth(200)
                playerBank.cnt:Show()

                playerBank.scrollFrame:SetScrollChild(playerBank.cnt)
                -- Grab >= from guild roster
                NS_WishLists_Master:getGuildRoster()
                local parsed_users = {}
                local found = false
                if not NS_WishLists_Master_Data_001["guild_roster"] then NS_WishLists_Master_Data_001["guild_roster"]={} end
                
                NS_WishLists_Master:ParseAllWLS()

                local all_wls = NS_WishLists_Master.allWLS()

                local temp_users = {}
                for itemid, prio_data in pairs(NS_WishLists_Master.Instance[instance_id]["items"]) do
                    if itemid then
                        local name, _, quality, iLevel, reqLevel, item_type, item_subType, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemid)
                        if name then
                            local prio_exists = false
                            for prio_name, prio_data_ in pairs(prio_data) do
                                if prio_name then
                                    prio_exists = true 
                                    break
                                end
                            end
                            if prio_exists then
                                for prio_name, prio_data_ in pairs(prio_data) do
                                    for _,user_name in ipairs(prio_data_) do
                                        temp_users[user_name] = true
                                    end
                                end
                            end
                        end
                    end
                end

                local user_names = {}
                local user_cpt = 0
                for user_name, _ in pairs(temp_users) do
                    if user_name then
                       -- print(user_name)
                        user_names[tostring(user_cpt)] = user_name
                        user_cpt = user_cpt + 1
                    end
                end


                for user_name, user in pairs(NS_WishLists_Master_Data_001["guild_roster"])  do
                    if user.user_name then
                        if #user.user_name >= 3 then
                            local userHaveWL = false
                            for k,user_name in pairs(user_names) do
                                if user.user_name == user_name then
                                    userHaveWL = true
                                    break
                                end
                            end
                            if userHaveWL then
                                local display_user = false
                                if rank_name then
                                    if rank_name == user.rank or rank_name == "All" then
                                        display_user = true
                                    end
                                else
                                    display_user = true
                                end
                                if display_user and user.level >= 58 then
                                    parsed_users[user.user_name] = {}
                                    parsed_users[user.user_name].name = user.user_name
                                    parsed_users[user.user_name].level = user.level
                                    parsed_users[user.user_name].class = user.user_classef
                                    parsed_users[user.user_name].rank = user.rank
                                    parsed_users[user.user_name].classColor = RAID_CLASS_COLORS[user.user_classef] or RAID_CLASS_COLORS["PRIEST"]
                                end
                            end
                        end
                    end
                end

                for user_name, user in pairs(NS_WishLists_Master_Data_001["receivedUsers"])  do
                    if user_name then
                        if #user_name >= 3 then
                            local userHaveWL = false
                            for k,user_namel in pairs(user_names) do
                                if user_name == user_namel then
                                    userHaveWL = true
                                    break
                                end
                            end
                            if userHaveWL and user["infos"].level >= 58 then
                                if not parsed_users[user_name] then
                                    parsed_users[user_name] = {}
                                end
                                parsed_users[user_name].name  = user_name;
                                parsed_users[user_name].level = user["infos"].level;
                                parsed_users[user_name].class = user["infos"]["classe"]["englishClass"];
                                parsed_users[user_name].race  = user["infos"].race;
                                parsed_users[user_name].classColor = RAID_CLASS_COLORS[user["infos"]["classe"]["englishClass"]] or RAID_CLASS_COLORS["PRIEST"]
                            end
                        end
                    end
                end
                local t_names = {}
                for k, v in pairs(parsed_users) do
                    tinsert(t_names, k)
                end
                table.sort(t_names)
                local cpt1 = 1
                for k, v in pairs(t_names) do
                    local userInfos = parsed_users[v]
                    local c = {}
                    if pair_ then
                        c = drk_lines
                    else
                        c = clr_lines
                    end
                    pair_ = not pair_
                    local playerFrame = CreateFrame("Button", addonname.."_GplayerFrame"..k, playerBank.cnt)
                    playerFrame:EnableMouse(true)
                    playerFrame:SetWidth(180)
                    playerFrame:SetHeight(20)
                    playerFrame:SetScript("OnClick", function(self)
                        NS_WishLists_Master.SelectedUsername = v;
                        NS_WishLists_Master:fillTargetWishList(instance_id,v)
                    end)

                    local labelplayer = playerFrame:CreateFontString(addonname.."_GplayerFrame_txt_"..k, "BACKGROUND", "GameFontHighlight")
                        labelplayer:SetPoint("TOPLEFT",0,0)
                        labelplayer:SetPoint("BOTTOMRIGHT",0,0)
                        labelplayer:SetJustifyH("CENTER")
                        labelplayer:SetJustifyV("MIDDLE")
                        local classColor = userInfos.classColor
                        labelplayer:SetTextColor(classColor.r, classColor.g, classColor.b)
                        labelplayer:SetText(v)
                        labelplayer:SetHeight(19)
                        labelplayer:SetWidth(180)
                        local highlight1 = playerFrame:CreateTexture(nil, "HIGHLIGHT")
                        highlight1:SetAllPoints(labelplayer)
                        highlight1:SetColorTexture(0.5, 0.5, 0.9, 0.8)
                        highlight1:SetBlendMode("ADD")
                        local highlight1bg = playerFrame:CreateTexture(nil, "BACKGROUND")
                        highlight1bg:SetAllPoints(labelplayer)
                        highlight1bg:SetColorTexture(c.r, c.g, c.b, c.a)
                        highlight1bg:SetBlendMode("ADD")
                    playerFrame:SetPoint("TOPLEFT", 0, -(20+2)*(k-1))
                end
            end
        end

        displayAll:SetScript("OnMouseUp", function(self)
            self:SetChecked(self:GetChecked());
            NS_WishLists_Master_Data_001["config"][instance_id]["displayAll"] = not NS_WishLists_Master_Data_001["config"][instance_id]["displayAll"]
            FillGuildRoster()
        end)
        FillGuildRoster()
        return playerBank, ItemsList
end

function NS_WishLists_Master:fillTargetWishList(instance_id, target_name)
    if NS_WishLists_Master.SelectedUsername and #NS_WishLists_Master.SelectedUsername >= 3 then
        target_name = NS_WishLists_Master.SelectedUsername
    end
    if not target_name  then return end
    local ItemsList = NS_WishLists_Master.ItemList

    if not ItemsList then 
        return
    end
    local obj_name = addonname.."_Instance_"..instance_id.."_MyList_cnt"
    if _G[obj_name] then
        _G[obj_name]:Hide()
        _G[obj_name] = nil
    end

    ItemsList.cnt = CreateFrame("Frame", addonname.."_Instance_"..instance_id.."_MyList_cnt", ItemsList.scrollFrame)
    ItemsList.cnt:SetPoint("TOPLEFT",ItemsList.scrollMotherFrame, "TOPLEFT", 0, 0)
    ItemsList.cnt:SetWidth(421)
    ItemsList.cnt:SetHeight(300)
    ItemsList.scrollFrame:SetScrollChild(ItemsList.cnt)

    if not ItemsList then print("error container is nil") end

    if not NS_WishLists_Master_Data_001["receivedWLS"] then 
        NS_WishLists_Master_Data_001["receivedWLS"] = {}
    end
    if not NS_WishLists_Master_Data_001["receivedWLS"][target_name] then 
        NS_WishLists_Master_Data_001["receivedWLS"][target_name] = {}
    end
    if not NS_WishLists_Master_Data_001["receivedWLS"][target_name][instance_id] then 
        NS_WishLists_Master_Data_001["receivedWLS"][target_name][instance_id] = {}
        return
    end
    pair_ = false
    local ItemsButtons = {}
    local prio_count = 0
    for k,v in pairs(NS_WishLists_Master_Data_001["receivedWLS"][target_name][instance_id]) do
        prio_count = prio_count + 1
    end
    --print(NS_WishLists_Master_Data_001["receivedWLS"][target_name][instance_id]["prio_1"])
    for index = 1,prio_count do
        local itemid = NS_WishLists_Master_Data_001["receivedWLS"][target_name][instance_id]["prio_"..index]
        if itemid then
            local name, _, quality, iLevel, reqLevel, item_type, item_subType, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemid)
            if name then
                local item_texture = GetItemIcon(itemid)
                local frameButton = CreateFrame("Button",nil,ItemsList.cnt)
                    frameButton:SetPoint("TOPLEFT", ItemsList.cnt, "TOPLEFT", 5, 32-((32*index-1)+2))
                    frameButton:SetPoint("RIGHT", ItemsList.cnt, "RIGHT", -5, 0)
                    frameButton:SetHeight(30)
                    frameButton:EnableMouse(false)
                    frameButton:SetScript("OnEnter", function(self)
                        PlaySoundFile("Sound/Interface/Iuiinterfacebuttona.Ogg");
                        GameTooltip:SetOwner(frameButton,"ANCHOR_CURSOR");
                        GameTooltip:ClearLines();
                        GameTooltip:SetItemByID(itemid);
                        GameTooltip:Show()
                        GameTooltip:SetItemByID(itemid);
                    end)
                    frameButton:SetScript("OnLeave", function(self)
                        GameTooltip:Hide()
                    end)

                    local Button_ = CreateFrame("Button",nil,frameButton)
                        Button_:SetWidth(30)
                        Button_:SetHeight(30)
                        Button_.icon = Button_:CreateTexture(nil,"ARTWORK");
                            Button_.icon:SetAllPoints();
                            Button_.icon:SetSize(28,28);
                            Button_.icon:SetTexture(item_texture);
                        Button_:SetPoint("TOPLEFT", frameButton, "TOPLEFT", 0, 0)
                --NAME
                    local labelItemName = frameButton:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
                        labelItemName:SetPoint("TOPLEFT", Button_,"TOPRIGHT",5, -5)
                        labelItemName:SetJustifyH("LEFT")
                        labelItemName:SetJustifyV("MIDDLE")
                        local r, g, b, hex = GetItemQualityColor(quality)
                        labelItemName:SetText("|c"..hex..name.."|r")
                        labelItemName:SetHeight(20)
                        labelItemName:SetWidth(300)

                    local ButtonAdd = CreateFrame("Button",nil,frameButton)
                        ButtonAdd:SetWidth(30)
                        ButtonAdd:SetHeight(30)
                        ButtonAdd:Raise()
                        ButtonAdd:SetToplevel(true)
                        ButtonAdd.icon = ButtonAdd:CreateTexture(nil,"ARTWORK");
                            ButtonAdd.icon:SetAllPoints(ButtonAdd);
                            ButtonAdd.icon:SetSize(28,28);
                            ButtonAdd.icon:SetTexture("Interface\\AddOns\\"..addonname.."\\Images\\remove_red.blp");
                        ButtonAdd:SetPoint("TOPLEFT", labelItemName,"TOPRIGHT",5, 5)
                        ButtonAdd:SetScript("OnClick", function(self)
                            local cpt_r = 0
                            local found = false
                            local index_ = 0
                            for k, v in pairs(NS_WishLists_Master_Data_001["receivedWLS"][target_name][instance_id]) do
                                cpt_r = cpt_r + 1
                                if v then
                                    if tonumber(v) == tonumber(itemid) then
                                        found = true
                                        index_ = k
                                    end
                                end
                            end
                            if found then
                                NS_WishLists_Master_Data_001["receivedWLS"][target_name][instance_id][index_] = nil
                            end
                            NS_WishLists_Master:fillTargetWishList(instance_id)
                        end)
                        local highlight = ButtonAdd:CreateTexture(nil, "HIGHLIGHT")
                            highlight:SetAllPoints(ButtonAdd)
                            highlight:SetTexture(136580) 
                            highlight:SetTexCoord(0, 1, 0.23, 0.77)
                            highlight:SetBlendMode("ADD")
                    local backdrop1 = {
                        bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
                        edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]], edgeSize = 16,
                        insets = { left = 4, right = 4, top = 4, bottom = 4 }
                    }
                    local itemPrioFrame = CreateFrame("Frame",nil,frameButton)
                        itemPrioFrame:SetPoint("TOPLEFT", ButtonAdd,"TOPRIGHT",5, 0)
                        itemPrioFrame:SetWidth(30)
                        itemPrioFrame:SetHeight(30)
                        itemPrioFrame:SetBackdrop(backdrop1)
                        itemPrioFrame:SetBackdropColor(0, 0, 0,0.5)
                        itemPrioFrame:SetBackdropBorderColor(0.8, 0, 0.8)

                    local labelItemprio = itemPrioFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
                        labelItemprio:SetPoint("TOPLEFT", itemPrioFrame,"TOPLEFT",0,0)
                        labelItemprio:SetJustifyH("CENTER")
                        labelItemprio:SetJustifyV("MIDDLE")
                        labelItemprio:SetText(index)
                        labelItemprio:SetHeight(30)
                        labelItemprio:SetWidth(30)

                frameButton.ButtonAdd = ButtonAdd
            end
        end
    end
end

local function IsUnique(itemid)
    
end