-- Author      : ketch
-- Create Date : 12/19/2020 7:52:16 PM


local addonname, NS_WishLists_Master = ...
local _G = getfenv(0)
local NS_WishLists_Master =_G.NS_WishLists_Master

local GiveMasterLoot, GetNumGroupMembers, GetItemInfo, GetMasterLootCandidate, CloseDropDownMenus = GiveMasterLoot, GetNumGroupMembers, GetItemInfo, GetMasterLootCandidate, CloseDropDownMenus
function NS_WishLists_Master:CheckInstance()
    local name, type, _, _, maxPlayers, _, _, instanceMapId = GetInstanceInfo()
    local isRaid =  (type == 'raid') and true or false
    if not isRaid then
        return
    end
    if not name then
        return
    end
    local currZone = {}
    currZone.zone_id = instanceMapId    
    currZone.name = NS_WishLists_Master.Instances[instanceMapId].name or "NC"
    currZone.i_raid = isRaid
    currZone.maxPlayers = maxPlayers
    return currZone
end
function NS_WishLists_Master:AddLootToLog(lootRecord)
    if not NS_WishLists_Master_Data_001["loots_log"] then
        NS_WishLists_Master_Data_001["loots_log"] = {}
    end
    tinsert(NS_WishLists_Master_Data_001["loots_log"], lootRecord)
end

function NS_WishLists_Master:LogLootDrop(player, itemLink, quality, instanceID, bossID)
    if IsInRaid() then
        local itemId = NS_WishLists_Master:getItemId(itemLink)
        if instanceID and itemId then
            if NS_WishLists_Master.Instances[instanceID] then
                if NS_WishLists_Master.Instances[instanceID][itemId] then
                    if not bossID or not NS_WishLists_Master.Instances[instanceID][bossID] then
                        bossID = 0
                    end
                    --NS_WishLists_Master:Broadcast_Loot(itemId, player)
                    local lootRecord = {
                        ["timestamp"] = GetServerTime(),
                        ["instanceID"] = instanceID,
                        ["player"] = player,
                        ["itemLink"] = itemLink,
                        ["itemID"] = itemId,
                        ["quality"] = quality,
                        ["bossID"] = bossID
                    }
                    NS_WishLists_Master:AddLootToLog(lootRecord)
                end
            end
        end
    end

end
function NS_WishLists_Master:COMBAT_LOG_EVENT_UNFILTERED(...)
    local timestamp, combatEvent, _, _, _, _, _, destGUID, destName, _, _, spellID = CombatLogGetCurrentEventInfo();
    if combatEvent == "UNIT_DIED" then
        local bossID = 0;
        local localBossName = destName;
        local unitType, _, _, instanceID, zoneID, ID = strsplit("-", destGUID); --[Unit type]-0-[server ID]-[instance ID]-[zone UID]-[ID]-[Spawn UID]
        if (unitType == "Creature") or (unitType == "Vehicle") then
            bossID = tonumber(ID);
        end
        NS_WishLists_Master.lastBoss = bossID
    end
end
function NS_WishLists_Master:CHAT_MSG_LOOT(...)
    local event, lootstring, altPlayer,  _, _, player, specialFlags, zoneChannelID, channelIndex, channelBaseName  = ... --"text", "playerName", "languageName", "channelName", "playerName2", "specialFlags", zoneChannelID, channelIndex, "channelBaseName", unused, lineID, "guid", bnSenderID, isMobile, isSubtitle, hideSenderInLetterbox, supressRaidIcons
    local itemLink = string.match(lootstring,"|%x+|Hitem:.-|h.-|h|r")
    if itemLink then
        local itemString = string.match(itemLink, "item[%-?%d:]+")
        local itemId = NS_WishLists_Master:getItemId(itemLink)
        local curr_item = GetItemSavedInfos(itemId)
        if string.len(player) >= 1 then
            local currZone = NS_WishLists_Master:CheckInstance()
            local instanceMapID = 0
            if currZone then
                instanceMapID = currZone.zone_id
            end
            if curr_item.quality > 1 then
                NS_WishLists_Master:LogLootDrop(player, itemLink, curr_item.quality, instanceMapID, NS_WishLists_Master.lastBoss)
                NS_WishLists_Master:Print(player.." reçoit le butin: "..itemLink..". ID: "..itemId..".")
                --NS_WishLists_Master:Broadcast_Loot(itemId, player)
            end
        end
    end
end



--*******UI
local clr_lines = {
    r =0.2,
    g = 0.2,
    b = 0.4,
    a = 0.2
}
local drk_lines = {
    r =0.4,
    g = 0.4,
    b = 0.8,
    a = 0.2
}

function NS_WishLists_Master:DrawTabLooteditems(container)
    -- **************************************************
    -- *                    frameWL               *
    -- **************************************************
    -- create a scrollframe inside wl_frame_rslt
    local LooteditemsRecapsFrame = NS_WishLists_Master.AceGUI:Create("SimpleFrame")
    LooteditemsRecapsFrame:SetPoint("TOPLEFT", 0, 0)
    LooteditemsRecapsFrame:SetFullWidth(true)
    LooteditemsRecapsFrame:SetLayout("Fill")
    LooteditemsRecapsFrame:Show()
    container:AddChild(LooteditemsRecapsFrame)
    -- Create the TabGroup
    local tabLIRecaps = NS_WishLists_Master.AceGUI:Create("TabGroup")
    tabLIRecaps:SetLayout("Flow")
    -- Setup which tabs to show
    tabLIRecaps:SetTabs({
        { text = "Molten Core"      , value = "tabLootedItems1" },
        { text = "Onyxia"           , value = "tabLootedItems2" },
        { text = "Blackwing Lair"   , value = "tabLootedItems3" },
        { text = "Zul'Gurub"        , value = "tabLootedItems4" },
        { text = "Ahn Quiraj 20"    , value = "tabLootedItems5" },
        { text = "Ahn Quiraj 40"    , value = "tabLootedItems6" },
        { text = "Naxxramas"        , value = "tabLootedItems7" },
        { text = "All"              , value = "tabLootedItems8" }
    })
    -- Register callback
    tabLIRecaps:SetCallback("OnGroupSelected", NS_WishLists_Master.SelectGroup2)
    -- Set initial Tab (this will fire the OnGroupSelected callback)
    tabLIRecaps:SelectTab("tabLootedItems7")
    -- add to the frame container
    LooteditemsRecapsFrame:AddChild(tabLIRecaps)
end

function NS_WishLists_Master:DrawLootedItemRecapFrame(container, instance_id)
    if instance_id == 1 then instance_id = nil end 
    --TabLootsLogs
    local idex = 1
    if instance_id then
        idex = instance_id
    end
    if  _G[addonname.."tabLootedItems_"..idex] then
        _G[addonname.."tabLootedItems_"..idex]:Hide()
        _G[addonname.."tabLootedItems_"..idex] = {}
        _G[addonname.."tabLootedItems_"..idex] = nil
    end

    local TabLootsLogs = CreateFrame("FRAME", addonname.."tabLootedItems_"..idex, container.content)
        TabLootsLogs:SetPoint("TOPLEFT",0,0)
        TabLootsLogs:SetPoint("BOTTOMRIGHT",0,0)

    local c = drk_lines
    local labelwhenH = TabLootsLogs:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
        labelwhenH:SetPoint("TOPLEFT",10,-4)
        labelwhenH:SetJustifyH("CENTER")
        labelwhenH:SetJustifyV("MIDDLE")
        labelwhenH:SetText("Date")
        labelwhenH:SetHeight(19)
        labelwhenH:SetWidth(150)
        local highlight1h = TabLootsLogs:CreateTexture(nil, "HIGHLIGHT")
        highlight1h:SetAllPoints(labelwhenH)
        highlight1h:SetColorTexture(0.5, 0.5, 0.9, 0.8)
        highlight1h:SetBlendMode("ADD")
        local highlight1bgh = TabLootsLogs:CreateTexture(nil, "BACKGROUND")
        highlight1bgh:SetAllPoints(labelwhenH)
        highlight1bgh:SetColorTexture(c.r, c.g, c.b, c.a)
        highlight1bgh:SetBlendMode("ADD")
    
    local labelwhereH = TabLootsLogs:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
    labelwhereH:Hide()
    if not instance_id then
            labelwhereH:Show()
            labelwhereH:SetPoint("TOPLEFT",labelwhenH,"TOPRIGHT", 5,0)
            labelwhereH:SetJustifyH("CENTER")
            labelwhereH:SetJustifyV("MIDDLE")
            labelwhereH:SetText("Zone")
            labelwhereH:SetHeight(19)
            labelwhereH:SetWidth(100)
            local highlight2h = TabLootsLogs:CreateTexture(nil, "HIGHLIGHT")
            highlight2h:SetAllPoints(labelwhereH)
            highlight2h:SetColorTexture(0.5, 0.5, 0.9, 0.8)
            highlight2h:SetBlendMode("ADD")
            local highlight2bgh = TabLootsLogs:CreateTexture(nil, "BACKGROUND")
            highlight2bgh:SetAllPoints(labelwhereH)
            highlight2bgh:SetColorTexture(c.r, c.g, c.b, c.a)
            highlight2bgh:SetBlendMode("ADD")
    end

    local imageH = TabLootsLogs:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
        if not instance_id then
            imageH:SetPoint("TOPLEFT",labelwhereH,"TOPRIGHT", 5,0)
        else
            imageH:SetPoint("TOPLEFT",labelwhenH,"TOPRIGHT", 5,0)
        end
        imageH:SetJustifyH("CENTER")
        imageH:SetJustifyV("MIDDLE")
        imageH:SetText("Icon")
        imageH:SetHeight(19)
        imageH:SetWidth(24)
        local highlight3h = TabLootsLogs:CreateTexture(nil, "HIGHLIGHT")
        highlight3h:SetAllPoints(imageH)
        highlight3h:SetColorTexture(0.5, 0.5, 0.9, 0.8)
        highlight3h:SetBlendMode("ADD")
        local highlight3bgh = TabLootsLogs:CreateTexture(nil, "BACKGROUND")
        highlight3bgh:SetAllPoints(imageH)
        highlight3bgh:SetColorTexture(c.r, c.g, c.b, c.a)
        highlight3bgh:SetBlendMode("ADD")

    local labelwhath = TabLootsLogs:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
        labelwhath:SetPoint("TOPLEFT",imageH,"TOPRIGHT",5,0)
        labelwhath:SetJustifyH("CENTER")
        labelwhath:SetJustifyV("MIDDLE")
        labelwhath:SetText("Nom de l'objet")
        labelwhath:SetHeight(19)
        labelwhath:SetWidth(300)
        local highlight4 = TabLootsLogs:CreateTexture(nil, "HIGHLIGHT")
        highlight4:SetAllPoints(labelwhath)
        highlight4:SetColorTexture(0.5, 0.5, 0.9, 0.8)
        highlight4:SetBlendMode("ADD")
        local highlight4bg = TabLootsLogs:CreateTexture(nil, "BACKGROUND")
        highlight4bg:SetAllPoints(labelwhath)
        highlight4bg:SetColorTexture(c.r, c.g, c.b, c.a)
        highlight4bg:SetBlendMode("ADD")

    local labelbyh = TabLootsLogs:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
        labelbyh:SetPoint("TOPLEFT",labelwhath,"TOPRIGHT", 5,0)
        labelbyh:SetJustifyH("CENTER")
        labelbyh:SetJustifyV("MIDDLE")
        labelbyh:SetText("Joueur")
        labelbyh:SetHeight(19)
        labelbyh:SetWidth(100)
        local highlight5 = TabLootsLogs:CreateTexture(nil, "HIGHLIGHT")
        highlight5:SetAllPoints(labelbyh)
        highlight5:SetColorTexture(0.5, 0.5, 0.9, 0.8)
        highlight5:SetBlendMode("ADD")
        local highlight5bg = TabLootsLogs:CreateTexture(nil, "BACKGROUND")
        highlight5bg:SetAllPoints(labelbyh)
        highlight5bg:SetColorTexture(c.r, c.g, c.b, c.a)
        highlight5bg:SetBlendMode("ADD")

    local resetNutton = CreateFrame("Button", "wa5_rstlootlg_butt", TabLootsLogs, "UIPanelButtonTemplate")
        resetNutton:SetPoint("TOPLEFT",labelbyh,"TOPRIGHT",0,0)
        resetNutton:SetHeight(19)
        resetNutton:SetWidth(40)
        resetNutton:SetText("Reset")
        resetNutton:SetScript("OnClick", function(self)
            NS_WishLists_Master_Data_001["loots_log"] = {}

            NS_WishLists_Master.SelectGroupOthers(container, "OnClick", "tab_others_6")
        end)
        -- ScrollingMessageFrame
    local scrollMotherFrame = CreateFrame("Frame", nil, TabLootsLogs)
        scrollMotherFrame:SetPoint("TOPLEFT", 0, -25)
        scrollMotherFrame:SetPoint("BOTTOMRIGHT", -60, 10)
    local backdrop = {
        bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
        edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]], edgeSize = 16,
        insets = { left = 4, right = 3, top = 4, bottom = 3 }
    }
    scrollMotherFrame:SetBackdrop(backdrop)
    scrollMotherFrame:SetBackdropColor(0, 0, 0.1)
    scrollMotherFrame:SetBackdropBorderColor(0.8, 0.8, 0.8)

    TabLootsLogs.scrollMotherFrame = scrollMotherFrame

    local scrollFrame = CreateFrame("ScrollFrame", nil, scrollMotherFrame, "UIPanelScrollFrameTemplate")
        scrollFrame:SetPoint("TOPLEFT", 5, -8)
        scrollFrame:SetPoint("BOTTOMRIGHT", -30, 5)
        scrollFrame:EnableMouseWheel(true)
        scrollFrame:EnableMouse(true)

    TabLootsLogs.scrollFrame = scrollFrame

    local scrollbg2 = scrollFrame:CreateTexture(nil, "BACKGROUND")
        scrollbg2:SetAllPoints(true)
        scrollbg2:SetColorTexture(0, 0, 0, 0.9)

    TabLootsLogs.scrollbg2 = scrollbg2

    TabLootsLogs.cnt = CreateFrame("Frame", "fugo", f)
    TabLootsLogs.cnt:SetPoint("TOPLEFT",scrollFrame, "TOPLEFT", 0, 0)
    TabLootsLogs.cnt:SetPoint("TOPRIGHT", scrollFrame, "TOPRIGHT", 0, 0)
    TabLootsLogs.cnt:SetHeight(400)
    TabLootsLogs.cnt:SetWidth(700)

    scrollFrame:SetScrollChild(TabLootsLogs.cnt)

    local scrollbg3 = TabLootsLogs.cnt:CreateTexture(nil, "BACKGROUND")
        scrollbg3:SetAllPoints(true)
        scrollbg3:SetColorTexture(0, 0, 0, 0.9)

    local loots_log_count = 0

    pair_ = false
    if not NS_WishLists_Master_Data_001["loots_log"] then
        NS_WishLists_Master_Data_001["loots_log"] = {}
    end
    for idxloot , loot in ipairs(NS_WishLists_Master_Data_001["loots_log"]) do
        -- si le log n'a pas de propriété de type
       -- print (loot.itemLink)
        if loot.instanceID then
            if not loot.itemType or not loot.itemSubType or type(loot.itemID) == "string" or loot.itemID == -1 then
                local itemName, itemLink, itemRarity, itemLevel, itemMinLevel,
                itemType, itemSubType, itemStackCount, itemEquipLoc,
                itemIcon, itemSellPrice, itemClassID, itemSubClassID,
                bindType, expacID, itemSetID, isCraftingReagent = GetItemInfo(loot.itemLink)
                loot.itemType = itemType or "Divers"
                loot.itemSubType = itemSubType or "NC"
                loot.itemID =  tonumber(NS_WishLists_Master:getItemId(itemLink))
            end
            --don't log non monitored item type and extra items id
            local item_to_dispaly = false;
            --for k,v in pairs(NS_WishLists_Master.ITEMS_TYPE_MONITORED) do
            --    if loot.itemType == v then
            --        item_to_dispaly = true;
            --        --break
            --    end
            --end
            --for k,v in pairs(NS_WishLists_Master.ITEMS_EXTRA_MONITORED) do
            --    if tonumber(loot.itemID) == tonumber(v) then
            --        item_to_dispaly = true;
            --        --break
            --    end
            --end
            if instance_id and instance_id == loot.instanceID and NS_WishLists_Master.Instances[loot.instanceID] then
                item_to_dispaly = true;
            end
            if not instance_id then
                item_to_dispaly = true;
            end
            --****
            --print (loot.itemLink.." itemId"..loot.itemID.." Type: "..loot.itemType.." item_to_dispaly: "..tostring(item_to_dispaly))
            if item_to_dispaly == true then
                local c = {}
                if pair_ then
                    c = drk_lines
                else
                    c = clr_lines
                end

                pair_ = not pair_
                local lootframe = CreateFrame("Button", addonname.."_lootlogbutt"..loots_log_count, TabLootsLogs.cnt)
                    lootframe:EnableMouse(true)
                    lootframe:RegisterForClicks("LeftButtonUp", "RightButtonUp");
                    lootframe:SetWidth(700)
                    lootframe:SetHeight(28)
                    lootframe:SetScript("OnClick", function(self, button)
                        local alt_key = IsAltKeyDown()
                        local shift_key = IsShiftKeyDown()
                        local control_key = IsControlKeyDown()

                        if button == "LeftButton" then
                            if alt_key then
                                --NS_WishLists_Master:DisplayLootLogAnnounceFrame(loot)
                            elseif shift_key then
                                SendChatMessage(loot.player.." a reçu le butin: "..loot.itemLink..". ItemId: "..loot.itemID..".", "RAID");
                            elseif control_key then
                                SendChatMessage(loot.player.." a rendu le butin: "..loot.itemLink..". ItemId: "..loot.itemID..".", "RAID");
                            else -- No modifier key is pressed

                            end
                        elseif button == "RightButton" then
                            if alt_key then
                                --print("alt")
                                NS_WishLists_Master_Data_001["loots_log"][idxloot] = {}
                                NS_WishLists_Master.SelectGroup2(container, "OnClick", "tab_others_7")
                            elseif shift_key then
                                SendChatMessage(loot.player.." a reçu le butin: "..loot.itemLink..". ItemId: "..loot.itemID..".", "OFFICER");
                            elseif control_key then
                                SendChatMessage(loot.player.." a rendu le butin: "..loot.itemLink..". ItemId: "..loot.itemID..".", "OFFICER");
                            else -- No modifier key is pressed

                            end
                        else -- Handle MiddleButton

                        end
                    end)

                local labelwhen = lootframe:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
                    labelwhen:SetPoint("TOPLEFT",5,-4)
                    labelwhen:SetJustifyH("CENTER")
                    labelwhen:SetJustifyV("MIDDLE")
                    labelwhen:SetText(date("%d/%m/%Y %H:%M:%S",loot.timestamp))
                    labelwhen:SetHeight(19)
                    labelwhen:SetWidth(150)
                    local highlight1 = lootframe:CreateTexture(nil, "HIGHLIGHT")
                    highlight1:SetAllPoints(labelwhen)
                    highlight1:SetColorTexture(0.5, 0.5, 0.9, 0.8)
                    highlight1:SetBlendMode("ADD")
                    local highlight1bg = lootframe:CreateTexture(nil, "BACKGROUND")
                    highlight1bg:SetAllPoints(labelwhen)
                    highlight1bg:SetColorTexture(c.r, c.g, c.b, c.a)
                    highlight1bg:SetBlendMode("ADD")

                    local labelwhere = lootframe:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
                    labelwhere:Hide()
                if not instance_id then
                    labelwhere:Show()
                        labelwhere:SetPoint("TOPLEFT",labelwhen,"TOPRIGHT", 5,0)
                        labelwhere:SetJustifyH("CENTER")
                        labelwhere:SetJustifyV("MIDDLE")
                        local instance_name = "NC"
                        if loot.instanceID == 0 or loot.instanceID == "NC" then
                            instance_name = "NC"
                        else
                            if NS_WishLists_Master.Instances[loot.instanceID] then 
                                instance_name = NS_WishLists_Master.Instances[loot.instanceID].name
                            else
                                instance_name = "NC"
                            end
                        end
                        labelwhere:SetText(instance_name)
                        labelwhere:SetHeight(19)
                        labelwhere:SetWidth(100)
                        local highlight2 = lootframe:CreateTexture(nil, "HIGHLIGHT")
                        highlight2:SetAllPoints(labelwhere)
                        highlight2:SetColorTexture(0.5, 0.5, 0.9, 0.8)
                        highlight2:SetBlendMode("ADD")
                        local highlight2bg = lootframe:CreateTexture(nil, "BACKGROUND")
                        highlight2bg:SetAllPoints(labelwhere)
                        highlight2bg:SetColorTexture(c.r, c.g, c.b, c.a)
                        highlight2bg:SetBlendMode("ADD")
                end
                local image = CreateFrame("Button",nil,lootframe);
                    image:SetSize(24,24);
                    image:EnableMouse(false);
                    image.index = x;
                    image.icon = image:CreateTexture(nil,"ARTWORK");
                    image.icon:SetAllPoints();
                    image.icon:SetSize(24,24);
                    image.icon:SetTexture(GetItemIcon(NS_WishLists_Master:getItemId(loot.itemLink)))
                    image:SetScript("OnEnter", function(self)
                            GameTooltip:SetOwner(self,"ANCHOR_BOTTOMRIGHT");
                            GameTooltip:ClearLines();
                            GameTooltip:SetItemByID(NS_WishLists_Master:getItemId(loot.itemLink));
                            GameTooltip:Show()
                            GameTooltip:Show();
                    end)
                    image:SetScript("OnLeave", function(self)
                        GameTooltip:Hide();
                    end)
                    if not instance_id then
                        image:SetPoint("TOPLEFT", labelwhere,"TOPRIGHT", 5, 2)
                    else
                        image:SetPoint("TOPLEFT", labelwhen,"TOPRIGHT", 5, 2)
                    end

                local labelwhat = lootframe:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
                    labelwhat:SetPoint("TOPLEFT",image,"TOPRIGHT",5,-2)
                    labelwhat:SetJustifyH("LEFT")
                    labelwhat:SetJustifyV("MIDDLE")
                    local itemName = GetItemInfo(loot.itemLink)
                    labelwhat:SetText(itemName)
                    labelwhat:SetHeight(19)
                    labelwhat:SetWidth(300)
                    local highlight4 = lootframe:CreateTexture(nil, "HIGHLIGHT")
                    highlight4:SetAllPoints(labelwhat)
                    highlight4:SetColorTexture(0.5, 0.5, 0.9, 0.8)
                    highlight4:SetBlendMode("ADD")
                    local highlight4bg = lootframe:CreateTexture(nil, "BACKGROUND")
                    highlight4bg:SetAllPoints(labelwhat)
                    highlight4bg:SetColorTexture(c.r, c.g, c.b, c.a)
                    highlight4bg:SetBlendMode("ADD")

                local labelby = lootframe:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
                    labelby:SetPoint("TOPLEFT",labelwhat,"TOPRIGHT", 5,0)
                    labelby:SetJustifyH("CENTER")
                    labelby:SetJustifyV("MIDDLE")
                    labelby:SetText(loot.player)
                    labelby:SetHeight(19)
                    labelby:SetWidth(100)
                    local highlight5 = lootframe:CreateTexture(nil, "HIGHLIGHT")
                    highlight5:SetAllPoints(labelby)
                    highlight5:SetColorTexture(0.5, 0.5, 0.9, 0.8)
                    highlight5:SetBlendMode("ADD")
                    local highlight5bg = lootframe:CreateTexture(nil, "BACKGROUND")
                    highlight5bg:SetAllPoints(labelby)
                    highlight5bg:SetColorTexture(c.r, c.g, c.b, c.a)
                    highlight5bg:SetBlendMode("ADD")

                lootframe:SetPoint("TOPLEFT", 0, -(28+2)*loots_log_count)
                local userbg = lootframe:CreateTexture(nil, "BACKGROUND")
                    userbg:SetAllPoints(lootframe)
                    userbg:SetColorTexture(0.8, 0.8, 0.9, 0)

                loots_log_count = loots_log_count + 1
            end
        end
    end
    container.content:Show()
    return TabLootsLogs
end