-- Author      : ketch
-- Create Date : 12/10/2020 12:05:39 PM


local addonname, NS_WishLists_Master = ...
local _G = getfenv(0)
local NS_WishLists_Master =_G.NS_WishLists_Master

local listWidth = 340
local wishersFWidth = 95
local separator_width = 2
local lineHeight = 34

local fullPageWidth = function(listWidth_,separator_width_,wishersFWidth_)
    if not listWidth_ then listWidth_ = listWidth end
    if not separator_width_ then separator_width_ = separator_width end
    if not wishersFWidth_ then wishersFWidth_ = wishersFWidth end
    return listWidth_ + separator_width_ + wishersFWidth_
end
function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k,v in pairs(t) do
        keys[#keys+1] = v
    end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end
function NS_WishLists_Master:DrawWLSRecapFrame(container, instance_id, filter)
    if not filter then filter = "itemID" end 
    if _G[addonname.."_RecapFrame"] then
        _G[addonname.."_RecapFrame"]:Hide()
        _G[addonname.."_RecapFrame"] = nil
    end
    
    local RecapFrame = CreateFrame("Frame",addonname.."_RecapFrame", container.content)
    RecapFrame:SetPoint("TOPLEFT",0,0)
    RecapFrame:SetPoint("BOTTOMRIGHT",0,0)
    RecapFrame:SetToplevel(true);
    local scrollbgmain = RecapFrame:CreateTexture(nil, "BACKGROUND")
    scrollbgmain:SetAllPoints(RecapFrame)
    scrollbgmain:SetColorTexture(0, 0, 0, 0.3)

    local c = NS_WishLists_Master.drk_lines

    local backdrop1 = {
        bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
        edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]], edgeSize = 16,
        insets = { left = 4, right = 4, top = 4, bottom = 4 }
    }
    -- ScrollingMessageFrame
    local scrollMotherFrame = CreateFrame("Frame", addonname.."_RecapFrameScrollMotherFrame", RecapFrame)
    scrollMotherFrame:SetPoint("TOPLEFT",  0, 0)
    scrollMotherFrame:SetPoint("BOTTOMRIGHT", 0, 0)
    scrollMotherFrame:SetBackdrop(backdrop1)
    scrollMotherFrame:SetBackdropColor(0, 0, 0, 0)
    scrollMotherFrame:SetBackdropBorderColor(0.8, 0, 0.8)

    local scrollFrame = CreateFrame("ScrollFrame", addonname.."_RecapFrameScrollFrame", scrollMotherFrame)--, "UIPanelScrollFrameTemplate")
    scrollFrame:SetPoint("TOPLEFT", 5, -8)
    scrollFrame:SetPoint("BOTTOMRIGHT", -30, 30)
    scrollFrame:EnableMouseWheel(true)
    scrollFrame:EnableMouse(true)

    local h_scrollBar = NS_WishLists_Master:CreateScrollBar(scrollFrame, true)
    h_scrollBar:SetPoint("BOTTOMLEFT",scrollFrame ,"BOTTOMLEFT", 3, -20)
    h_scrollBar:SetPoint("BOTTOMRIGHT",scrollFrame ,"BOTTOMRIGHT", 0, -20)
    h_scrollBar:SetMinMaxValues(0,500)
    h_scrollBar.hori = true

    local v_scrollBar = NS_WishLists_Master:CreateScrollBar(scrollFrame, false)
    v_scrollBar:SetPoint("TOPRIGHT",scrollFrame ,"TOPRIGHT", 20, 0)
    v_scrollBar:SetPoint("BOTTOMRIGHT",scrollFrame ,"BOTTOMRIGHT", 20, 0)
    v_scrollBar:SetMinMaxValues(0,500)
    v_scrollBar.hori = false
    --scrollbar bg
    local scrollbg2 = scrollFrame:CreateTexture(nil, "BACKGROUND")
    scrollbg2:SetAllPoints(v_scrollBar)
    scrollbg2:SetColorTexture(0, 0, 0, 0.9)

    local scrollbg3 = scrollFrame:CreateTexture(nil, "BACKGROUND")
    scrollbg3:SetAllPoints(h_scrollBar)
    scrollbg3:SetColorTexture(0, 0, 0, 0.7)

    RecapFrame.cnt = CreateFrame("Frame", addonname.."_RecapFrame_cnt", scrollFrame)
    RecapFrame.cnt:SetPoint("TOPLEFT",scrollMotherFrame, "TOPLEFT", 0, 0)
    RecapFrame.cnt:SetHeight(lineHeight)
    RecapFrame.cnt:SetScript("OnMouseWheel", function(self, arg1)
        if IsShiftKeyDown() then
            h_scrollBar._bar:SetValue(h_scrollBar._bar:GetValue() - arg1 * h_scrollBar._bar:GetValueStep() * 20);
        else
            v_scrollBar._bar:SetValue(v_scrollBar._bar:GetValue() - arg1 * v_scrollBar._bar:GetValueStep() * 20);
        end
    end);
    local userCellWidth = wishersFWidth;
    local userCellSeparator = 2;
    local index = 2
    local itemid = 22722
    NS_WishLists_Master:ParseAllWLS()

    local all_wls = NS_WishLists_Master.allWLS()
    local temp_users = {}

    for itemid, prio_data in pairs(NS_WishLists_Master.Instance[instance_id]["items"]) do
        if itemid then
            local name, _, quality, iLevel, reqLevel, item_type, item_subType, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemid)
            if name then
                local prio_exists = false
                for prio_name, prio_data_ in pairs(prio_data) do
                    if prio_name then
                        prio_exists = true 
                        break
                    end
                end
                if prio_exists then
                    for prio_name, prio_data_ in pairs(prio_data) do
                        for _,user_name in ipairs(prio_data_) do
                            temp_users[user_name] = true
                        end
                    end
                end
            end
        end
    end

    local user_names = {}
    local user_cpt = 0

    for user_name, _ in pairs(temp_users) do
        if user_name then
            user_names[tostring(user_cpt)] = user_name

            user_cpt = user_cpt + 1
        end
    end
    RecapFrame.cnt:SetWidth(listWidth + separator_width + wishersFWidth*(user_cpt) + separator_width*(user_cpt)+9)
    
    local labelItemList = RecapFrame.cnt:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
        labelItemList:SetPoint("TOPLEFT", 10,-10)
        labelItemList:SetJustifyH("CENTER")
        labelItemList:SetJustifyV("MIDDLE")
        labelItemList:SetText("Tous Les Items Lootables")
        labelItemList:SetWidth(listWidth)
        labelItemList:SetHeight(19)
        local highlight1h = RecapFrame.cnt:CreateTexture(nil, "HIGHLIGHT")
        highlight1h:SetAllPoints(labelItemList)
        highlight1h:SetColorTexture(0.5, 0.5, 0.9, 0.8)
        highlight1h:SetBlendMode("ADD")
        local highlight1bgh = RecapFrame.cnt:CreateTexture(nil, "BACKGROUND")
        highlight1bgh:SetAllPoints(labelItemList)
        highlight1bgh:SetColorTexture(c.r, c.g, c.b, c.a)
        highlight1bgh:SetBlendMode("ADD")

    local frameSortItems = CreateFrame("Frame", addonname.."_RecapFrameframeSortItems", RecapFrame.cnt)
        frameSortItems:SetPoint("TOPLEFT" , labelItemList, "BOTTOMLEFT", 0,0)
        frameSortItems:SetPoint("TOPRIGHT", labelItemList,"BOTTOMRIGHT",0, 0)
        frameSortItems:SetHeight(26)

        local Button_ = CreateFrame("Button",nil,frameSortItems)
            Button_:SetWidth(22)
            Button_:SetHeight(22)
            Button_.icon = Button_:CreateTexture(nil,"ARTWORK");
                Button_.icon:SetAllPoints();
                Button_.icon:SetSize(28,28);
                Button_.icon:SetTexture("Interface\\AddOns\\"..addonname.."\\Images\\Sort-Arrow");
            Button_:SetPoint("TOPRIGHT", frameSortItems, "TOPRIGHT", -2, -2)
    
            Button_:SetScript("OnClick", function(self, button)
                container:ReleaseChildren()
                NS_WishLists_Master:HideAll()
                if filter == "itemID" then filter = "name" else filter = "itemID" end
                NS_WishLists_Master.WLSRecapFrame = NS_WishLists_Master:DrawWLSRecapFrame(container, instance_id,filter)
            end)
    
    local searchTxtBox = CreateFrame("EditBox", addonname.."_searchTxtBox", frameSortItems)
       searchTxtBox:SetPoint("TOPLEFT",0,-2)
       searchTxtBox:SetPoint("TOPRIGHT", Button_, "TOPLEFT", 0,-2)
       searchTxtBox:SetHeight(24)
       searchTxtBox:SetFont('Fonts\\ARIALN.ttf', 14, 'THINOUTLINE')
       searchTxtBox:SetFontObject(ChatFontNormal)
       searchTxtBox:SetMultiLine(false)
       searchTxtBox:EnableMouse(true)
       searchTxtBox:SetAutoFocus(false)
       searchTxtBox:SetCountInvisibleLetters(false)
       searchTxtBox:SetHyperlinksEnabled(true)
       searchTxtBox:SetBackdrop(backdrop1)
       searchTxtBox:SetBackdropColor(0, 0, 0,0.5)
       searchTxtBox:SetBackdropBorderColor(0.8, 0, 0.8)
       searchTxtBox:SetTextInsets(5,5,2,2)
    local labelWishers = RecapFrame.cnt:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
        labelWishers:SetPoint("TOPLEFT", labelItemList, "TOPRIGHT",separator_width,0)
        labelWishers:SetPoint("TOPRIGHT", RecapFrame.cnt, "TOPRIGHT", 0,0)
        labelWishers:SetJustifyH("LEFT")
        labelWishers:SetJustifyV("MIDDLE")
        labelWishers:SetText("  Raiders")
        labelWishers:SetHeight(19)

        labelWishers.highlight1h = RecapFrame.cnt:CreateTexture(nil, "HIGHLIGHT")
        labelWishers.highlight1h:SetAllPoints(labelWishers)
        labelWishers.highlight1h:SetColorTexture(0.5, 0.5, 0.9, 0.8)
        labelWishers.highlight1h:SetBlendMode("ADD")
        
        labelWishers.highlight1bgh = RecapFrame.cnt:CreateTexture(nil, "BACKGROUND")
        labelWishers.highlight1bgh:SetAllPoints(labelWishers)
        labelWishers.highlight1bgh:SetColorTexture(c.r, c.g, c.b, c.a)
        labelWishers.highlight1bgh:SetBlendMode("ADD")


        for user_cpt2 = 0, user_cpt-1 do
            user_name = user_names[tostring(user_cpt2)]
            local classColor = RAID_CLASS_COLORS["PRIEST"];
            if NS_WishLists_Master_Data_001["receivedUsers"][user_name] then
                classColor = RAID_CLASS_COLORS[NS_WishLists_Master_Data_001["receivedUsers"][user_name]["infos"]["classe"]["englishClass"]]
            end
            local FrameUserCol = CreateFrame("Frame", addonname.."_RecapFrameUser_"..user_name, RecapFrame.cnt)
            FrameUserCol:SetPoint("TOPLEFT", labelWishers, "BOTTOMLEFT", 0+((userCellWidth*(user_cpt2))+2), 0)
            FrameUserCol:SetPoint("BOTTOM", 0, 0)
            FrameUserCol:SetWidth(userCellWidth)
            FrameUserCol:SetBackdrop(backdrop1)
            FrameUserCol:SetBackdropColor(0, 0, 0, 0.8)
            FrameUserCol:SetBackdropBorderColor(classColor.r, classColor.g, classColor.b)
            local labelUserName = FrameUserCol:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
            labelUserName:SetPoint("TOPLEFT", labelWishers, "BOTTOMLEFT", 0+((userCellWidth*user_cpt2)+2), -5)
            labelUserName:SetJustifyH("CENTER")
            labelUserName:SetJustifyV("MIDDLE")
            labelUserName:SetText(user_name)
            labelUserName:SetTextColor(classColor.r, classColor.g, classColor.b)
            labelUserName:SetHeight(20)
            labelUserName:SetWidth(userCellWidth)
        end
        --**********************
        local items_data = {}
        --print(filter)
        for itemid, prio_data in pairs(NS_WishLists_Master.Instance[instance_id]["items"]) do
            if itemid then
                local curr_item = GetItemSavedInfos(itemid)
                if filter == "name" then 
                    items_data[itemid] = curr_item.name
                elseif filter == "itemID" then 
                    items_data[itemid] = curr_item.itemID
                end
            end
        end

        local keys = {}
        for k,v in pairs(items_data) do
            keys[#keys+1] = v
            --print("v: "..v)
        end
        table.sort(keys)
        local items_data2 = {}
        for k1,v1 in pairs(keys) do
            --print(k1,v1)
            for k2,v2 in pairs(items_data) do
                --print(k2)
                if v1 == v2 then
                    items_data2[#items_data2+1] = k2
                end

            end
        end
        for k1,v1 in pairs(items_data2) do
            --print(k1,v1)
        end
        --*******************
        for k1 , itemid in pairs(items_data2) do
            local prio_data = NS_WishLists_Master.Instance[instance_id]["items"][itemid]
            if itemid then
                local curr_item = GetItemSavedInfos(itemid)
                if curr_item.name then
                    local prio_exists = false
                    for prio_name, prio_data_ in pairs(prio_data) do
                        if prio_name then
                            prio_exists = true 
                            break
                        end
                    end
                    if prio_exists then
                        local r, g, b, hex = GetItemQualityColor(curr_item.quality)
                        RecapFrame.cnt:SetHeight(RecapFrame.cnt:GetHeight()+lineHeight)
                        v_scrollBar:SetMinMaxValues(0,RecapFrame.cnt:GetHeight())
                        local FrameItemLine = CreateFrame("Frame", addonname.."_RecapFrameItem_"..itemid, RecapFrame.cnt)
                            FrameItemLine:SetPoint("TOPLEFT", labelItemList, "BOTTOMLEFT", 0, -((lineHeight*(index-1))+2)+10)
                            FrameItemLine:SetPoint("RIGHT", RecapFrame.cnt ,"RIGHT",0, 0)
                            FrameItemLine:SetHeight(lineHeight)
                            FrameItemLine:SetBackdrop(backdrop1)
                            FrameItemLine:SetBackdropColor(0, 0, 0, 0.8)
                            FrameItemLine:SetBackdropBorderColor(r,g,b)

                        local item_texture = GetItemIcon(itemid)
                            local frameButton = CreateFrame("Button",nil, FrameItemLine)
                                frameButton:SetPoint("TOPLEFT", 2,-2)
                                frameButton:SetWidth(listWidth)
                                frameButton:SetHeight(30)
                                frameButton:EnableMouse(false)
                                frameButton:SetScript("OnEnter", function(self)
                                    PlaySoundFile("Sound/Interface/Iuiinterfacebuttona.Ogg");
                                    GameTooltip:SetOwner(frameButton,"ANCHOR_CURSOR");
                                    GameTooltip:ClearLines();
                                    GameTooltip:SetItemByID(itemid);
                                    GameTooltip:Show()
                                    GameTooltip:SetItemByID(itemid);
                                end)
                                frameButton:SetScript("OnLeave", function(self)
                                    GameTooltip:Hide()
                                end)

                                local Button_ = CreateFrame("Button",nil,frameButton)
                                    Button_:SetWidth(30)
                                    Button_:SetHeight(30)
                                    Button_.icon = Button_:CreateTexture(nil,"ARTWORK");
                                        Button_.icon:SetAllPoints();
                                        Button_.icon:SetSize(28,28);
                                        Button_.icon:SetTexture(item_texture);
                                    Button_:SetPoint("TOPLEFT", frameButton, "TOPLEFT", 0, 0)
                            --NAME
                                local labelItemName = frameButton:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
                                    labelItemName:SetPoint("TOPLEFT", Button_,"TOPRIGHT",5, -5)
                                    labelItemName:SetJustifyH("LEFT")
                                    labelItemName:SetJustifyV("MIDDLE")
                                    labelItemName:SetText("|c"..hex..curr_item.name.."|r")
                                    labelItemName:SetHeight(20)
                                    labelItemName:SetWidth(300)

                        for prio_name, prio_data_ in pairs(prio_data) do
                            local prio_num = string.match(prio_name, '%d+');
                            local column_index = 0;
                            for _,user_name in ipairs(prio_data_) do
                                for k,v in pairs(user_names) do
                                    if user_name == v then
                                        column_index = tonumber(k)
                                        local labelprioFrame = CreateFrame("Button",nil, FrameItemLine)
                                        labelprioFrame:SetPoint("TOPLEFT", frameButton, "TOPRIGHT", ((userCellWidth*(column_index))+2)-2, 2)
                                        labelprioFrame:SetHeight(lineHeight)
                                        labelprioFrame:SetWidth(userCellWidth)

                                        local labelItemPrioValue = labelprioFrame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
                                        labelItemPrioValue:SetPoint("TOPLEFT", 0,0)
                                        labelItemPrioValue:SetJustifyH("CENTER")
                                        labelItemPrioValue:SetJustifyV("MIDDLE")
                                        labelItemPrioValue:SetText(prio_num)
                                        labelItemPrioValue:SetHeight(lineHeight)
                                        labelItemPrioValue:SetWidth(userCellWidth)

                                        labelprioFrame.tooltip = CreateFrame("Frame", addonname.."_prio_Tooltip", labelprioFrame, "TooltipBorderedFrameTemplate");
                                        labelprioFrame.tooltip:SetPoint("TOPLEFT", labelprioFrame, "CENTER", 0, 30);
                                        labelprioFrame.tooltip:SetFrameStrata("TOOLTIP");
                                        labelprioFrame.tooltip:SetFrameLevel(9);
                                        labelprioFrame.tooltip.fs = labelprioFrame.tooltip:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
                                        labelprioFrame.tooltip.fs:SetPoint("CENTER", 0, 0);
                                        labelprioFrame.tooltip.fs:SetText("|CffDEDE42"..user_name.. "|r\n|cFFFFFFFF"..curr_item.name.. "|r")--);
                                        labelprioFrame.tooltip:SetWidth(labelprioFrame.tooltip.fs:GetStringWidth() + 18);
                                        labelprioFrame.tooltip:SetHeight(labelprioFrame.tooltip.fs:GetStringHeight() + 12);
                                        labelprioFrame:SetScript("OnEnter", function(self)
                                            labelprioFrame.tooltip:Show();
                                        end)
                                        labelprioFrame:SetScript("OnLeave", function(self)
                                            labelprioFrame.tooltip:Hide();
                                        end)
                                        labelprioFrame.lastUpdate = 0;
   
                                        --Make it act like pin is the parent and not WorldMapFrame.
                                        labelprioFrame:SetScript("OnHide", function(self)
                                        end)
                                        labelprioFrame:SetScript("OnShow", function(self)
                                        end)
                                        labelprioFrame.tooltip:Hide();
                                        labelprioFrame:SetScript("OnMouseDown", function(self, button)
                                            if (IsShiftKeyDown()) then

                                            end
                                        end)
                                    end
                                end
                            
                            end
                        end
                        index = index + 1
                    end
                end
            end
        end

    RecapFrame.cnt:SetHeight(RecapFrame.cnt:GetHeight()+24)

    scrollFrame:SetScrollChild(RecapFrame.cnt)

    RecapFrame.scrollFrame = scrollFrame
    RecapFrame.scrollMotherFrame = scrollMotherFrame
    _G[addonname.."_RecapFrame"] = RecapFrame
    return RecapFrame
end

function NS_WishLists_Master:CreateScrollBar(parent, horizontal)		
    local BARWIDTH = 19;
	local BUTTONWIDTH = BARWIDTH * 0.6;

	local frame = CreateFrame("Frame", nil, parent);

	frame:Show();
	frame:SetWidth(BARWIDTH);
	frame:SetHeight(BARWIDTH);
	frame:EnableMouseWheel(true); -- Mouse wheel support
	
	-- Creates a border to decorate the scroll bar
	local border = CreateFrame("Frame", nil, frame);
	border:SetBackdrop({ bgFile = "", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 12, insets = {left = 0, right = 0, top = 0, bottom = 0 }, });
	border:SetBackdropBorderColor(0.8, 0, 0.8, 0.5);
	border:SetAllPoints(frame);
	border:SetScale(0.75); -- Just to make the border soft and fine

	-- This is the real Slider
	local bar = CreateFrame("Slider", nil, frame);
	frame._bar = bar;		
    local horizontal_ = horizontal
	bar:SetOrientation("VERTICAL");	
	if horizontal then
		bar:SetOrientation("HORIZONTAL");	
	end

	bar:SetMinMaxValues(0, 0);
	bar:SetValueStep(1);
	bar:SetValue(0);

	bar:SetWidth(BUTTONWIDTH);
	bar:SetHeight(BUTTONWIDTH);			
	
	-- The thumb nail texture
	local thumb = bar:CreateTexture(nil, "OVERLAY");
	thumb:SetTexture("Interface\\ChatFrame\\ChatFrameBackground");
	
	if horizontal then
		thumb:SetGradientAlpha("VERTICAL", 0.15, 0.15, 0.15, 1, 0.5, 0.5, 0.5, 0.75)
	else
		thumb:SetGradientAlpha("HORIZONTAL", 0.5, 0.5, 0.5, 0.75, 0.15, 0.15, 0.15, 1)
	end
	
	thumb:SetWidth(BUTTONWIDTH);
	thumb:SetHeight(BUTTONWIDTH);
	bar:SetThumbTexture(thumb);

	-- Create scroll bar buttons
	bar.CreateButton = function(self, oper, horizontal)
		local button = CreateFrame("Button", nil, self);
		button.CreateButtonTexture = function(self, iconPath, horizontal, layer, add)
			local texture = self:CreateTexture(nil, layer or "ARTWORK");
			texture:SetHeight(BUTTONWIDTH * 2);
			texture:SetWidth(BUTTONWIDTH * 2);
			texture:SetTexture(iconPath);
			if horizontal then
				-- Blizzard folks did not even provide horizontal scroll bar button icons, I have to use their vertical ones
				-- and rotate them by 90 degrees, oh man, sorry about the following nasty code.					
				local SQRT2 = math.sqrt(2);
				texture:SetTexCoord(0.5 + math.cos(math.rad(315)) / SQRT2, 0.5 + math.sin(math.rad(315)) / SQRT2,
						    0.5 + math.cos(math.rad(225)) / SQRT2, 0.5 + math.sin(math.rad(225)) / SQRT2,
						    0.5 + math.cos(math.rad(45))  / SQRT2, 0.5 + math.sin(math.rad(45))  / SQRT2, 
						    0.5 + math.cos(math.rad(135)) / SQRT2, 0.5 + math.sin(math.rad(135)) / SQRT2);
			end

			if add then
				texture:SetBlendMode("ADD");
			end				

			texture:SetPoint("CENTER", self, "CENTER");
			return texture;
		end;

		button:SetWidth(BUTTONWIDTH);
		button:SetHeight(BUTTONWIDTH);
		button._oper = oper;

		local texturePrefix = "Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-";
		if oper == 1 then
			texturePrefix = "Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-";
		end

		button:SetNormalTexture(button:CreateButtonTexture(texturePrefix.."Up", horizontal));
		button:SetPushedTexture(button:CreateButtonTexture(texturePrefix.."Down", horizontal));
		button:SetDisabledTexture(button:CreateButtonTexture(texturePrefix.."Disabled", horizontal));
		button:SetHighlightTexture(button:CreateButtonTexture(texturePrefix.."Highlight", horizontal));

		-- Buttons shall be disabled is the Slider value hits its limit.
		button.UpdateStatus = function(self, value)
			value = value or self:GetParent():GetValue();
			local minVal, maxVal = self:GetParent():GetMinMaxValues();
			if (self._oper == -1 and value <= minVal) or (self._oper == 1 and value >= maxVal) then
				self:Disable();
			else
				self:Enable();
			end
		end;

		

		-- Clicks on scroll bar button increases or decreases Slider value, depending on which button is clicked.
		--button:SetScript("OnClick", function() this:OnClick(); end);

		button:SetScript("OnMouseDown", function(self)
			self:SetScript("OnUpdate", function(self, elapse)
				self._updateElapsed = (self._updateElapsed or 10) + elapse;
				if self._updateElapsed > 0.1 then
					self._updateElapsed = 0;
					local parent = self:GetParent();
					parent:SetValue(parent:GetValue() + self._oper * parent:GetValueStep());										
				end					
			end);
		end);

		button:SetScript("OnMouseUp", function(self)				
			self:SetScript("OnUpdate", nil);
			self._updateElapsed = nil;
			PlaySound("UChatScrollButton");
		end);			

		button:UpdateStatus();
		return button;
	end;

	bar.UpdateButtonStatus = function(self, value)
		value = value or self:GetValue();
		bar._button1:UpdateStatus(value);
		bar._button2:UpdateStatus(value);
	end;
	
	bar._button1 = bar:CreateButton(-1, horizontal);
	bar._button2 = bar:CreateButton(1, horizontal);		

	-- Setup the scroll bar layouts
	if horizontal then			
		bar._button1:SetPoint("LEFT", frame, "LEFT", BUTTONWIDTH / 4, 0);			
		bar._button2:SetPoint("RIGHT", frame, "RIGHT", -BUTTONWIDTH / 4, 0);
		bar:SetPoint("LEFT", frame, "LEFT", BUTTONWIDTH * 1.4, 0);
		bar:SetPoint("RIGHT", frame, "RIGHT", -BUTTONWIDTH * 1.4, 0);
	else			
		bar._button1:SetPoint("TOP", frame, "TOP", 0, -BUTTONWIDTH / 4);
		bar._button2:SetPoint("BOTTOM", frame, "BOTTOM", 0, BUTTONWIDTH / 4);
		bar:SetPoint("TOP", frame, "TOP", 0, -BUTTONWIDTH * 1.4);
		bar:SetPoint("BOTTOM", frame, "BOTTOM", 0, BUTTONWIDTH * 1.4);
	end

	bar.OnScrollBarUpdated = function(self)
		local parent = self:GetParent():GetParent();
		if parent and type(parent.OnScrollBarUpdated) == "function" then
			parent:OnScrollBarUpdated(self:GetParent());
		end
	end;		

	bar:SetScript("OnValueChanged", function(self,arg1)
		bar:UpdateButtonStatus(arg1);
        local parent = self:GetParent():GetParent():GetParent():GetParent().cnt
        local point, relativeTo, relativePoint, xOfs, yOfs = parent:GetPoint(1)

        if horizontal then
            parent:SetPoint(point,relativeTo, -bar:GetValue(),yOfs)
        else
            parent:SetPoint(point,relativeTo, xOfs, bar:GetValue())
        end
	end);	

	bar:SetScript("OnShow", function() bar:OnScrollBarUpdated(); end);
	bar:SetScript("OnHide", function() bar:OnScrollBarUpdated(); end);
	bar:SetScript("OnSizeChanged", function() bar:OnScrollBarUpdated(); end);
	
	-- Makes the scroll bar look like a Slider!
	frame.GetOrientation = function(self) return self._bar:GetOrientation(); end;		
	frame.GetValue = function(self) return self._bar:GetValue(); end;
	frame.SetValue = function(self, value) return self._bar:SetValue(value); end;		
	frame.GetValueStep = function(self) return self._bar:GetValueStep(); end;
	frame.SetValueStep = function(self, step) return self._bar:SetValueStep(step); end;
	frame.GetMinMaxValues = function(self) return self._bar:GetMinMaxValues(); end;

	frame.SetMinMaxValues = function(self, minValue, maxValue)
		-- Change the thumbnail texture size to fit the scroll range.
		local horizontal = self:GetOrientation() == "HORIZONTAL";
		local barSize, thumbSize;
		if horizontal then
			barSize = (self._bar:GetRight() or 0) - (self._bar:GetLeft() or 0);				
		else
			barSize = (self._bar:GetTop() or 0) - (self._bar:GetBottom() or 0);				
		end

		local range = maxValue - minValue;
		if range < barSize then				
			thumbSize = barSize - range;				
		else				
			thumbSize = barSize / math.max(range, 1) * BUTTONWIDTH;
		end
		
		thumbSize = math.max(6, math.min(barSize, thumbSize));
		
		if horizontal then
			self._bar:GetThumbTexture():SetWidth(thumbSize);
		else
			self._bar:GetThumbTexture():SetHeight(thumbSize);
		end
		
		self._bar:SetMinMaxValues(minValue, maxValue);
		self._bar:UpdateButtonStatus();
	end;

	frame.UpdateScrollRange = function(self, range)
		if type(range) ~= "number" then
			range = 0;
		end

		if range < 0.01 or range > 1000000 then
			range = 0;
		end

		-- Make the range evenly dividable by value-step, otherwise some bottom/right items might never
		-- be able to scroll up properly when value-step is greater than 1.
		range = math.ceil(range);
		local step = math.max(1, math.ceil(self:GetValueStep()));
		local md = range % step;
		if md > 0 then
			range = range - md + step;
		end
		
		self:SetMinMaxValues(0, range);
		if self:GetValue() >= range then
			self:SetValue(range);
		end
		
		return range > 0 and range;			
	end;

	frame:SetScript("OnMouseWheel", function(self, arg1) frame._bar:SetValue(frame._bar:GetValue() - arg1 * frame._bar:GetValueStep() * 5); end);
    
	return frame;
end
